package validatesitav1;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.Hashtable;
import org.sanskritlibrary.Regex;
import org.sanskritlibrary.SAXParser;
import org.sanskritlibrary.SAXParserHandler;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

public class ProcessAnnotations implements SAXParserHandler
{
    SAXParser saxParser;
    Hashtable<String, Boolean> manuscriptDirectories;
    String annotationsDirectory;
    String textDirectory;
    File currentFile;
    Regex directorynameRegex = new Regex("", "^(.+)/([^/]*)$");
    Regex filenameRegex = new Regex("", "^([^\\\\/]+)[\\\\/](.*)[\\\\/]([^\\\\/]+)$");
    Hashtable<String, Boolean> missing = new Hashtable<String, Boolean>();
    Hashtable<String, Boolean> mismatch = new Hashtable<String, Boolean>();
    Hashtable<String, Integer> annotationsCount = new Hashtable<String, Integer>();
    String dir;
    String legalPage;
    String xmlfile;

    public ProcessAnnotations (String annotationsDirectory, String textDirectory, Hashtable<String, Boolean> manuscriptDirectories) {
        saxParser = new SAXParser(this);
        this.annotationsDirectory = annotationsDirectory;
        this.manuscriptDirectories = manuscriptDirectories;
    }

    public void parse() throws Exception {
        process (new File(annotationsDirectory));
        Enumeration<String> en = annotationsCount.keys();
        ArrayList<AnnotationInfo> ais = new ArrayList<AnnotationInfo>();
        while (en.hasMoreElements()) {
            String key = en.nextElement();
            ais.add(new AnnotationInfo(key, annotationsCount.get(key)));
        }
        Collections.sort(ais, new AnnotationInfo());
        for (AnnotationInfo ai : ais) {
            System.out.format("%d. %s\n", ai.count, ai.filename);
        }
    }

    private void process (File in) throws Exception {
        File[] files = in.listFiles();
        for (File file : files) {
            if (file.isDirectory()) {
                process(file);
            } else {
                String filename = file.getName();
                if (filename.startsWith(".") || !filename.endsWith(".xml")) {
                    continue;
                }
                String subdirectory = file.getAbsolutePath().substring(annotationsDirectory.length());
                filenameRegex.reset(subdirectory);
                if (filenameRegex.find()) {
                    dir = filenameRegex.getGroup(2).replaceAll("\\\\", "/");
                    legalPage = String.format("%s/%s", filenameRegex.getGroup(1), dir);
                    xmlfile = filenameRegex.getGroup(3);
                    if (manuscriptDirectories.get(dir) == null) {
                        if (missing.get(dir) == null) {
                            System.out.format("%s is not defined in catalog.xml.\nFirst occurrence of error: %s\n\n",
                                    dir, file.getAbsolutePath());
                            missing.put(dir, true);
                        }
                    }
                } else {
                    System.out.format("Regular expression failed for %s\n", subdirectory);
                    continue;
                }

                currentFile = file;
                saxParser.parse(file.getAbsolutePath());
            }
        }

    }

    public void warning(SAXParseException exception) throws SAXException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void error(SAXParseException exception) throws SAXException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void fatalError(SAXParseException exception) throws SAXException {
        System.out.println(exception.getMessage());
        throw new SAXException("Boom!");
    }

    public void setDocumentLocator(Locator locator) {
    }

    public void startDocument() throws SAXException {
    }

    public void endDocument() throws SAXException {
    }

    public void startPrefixMapping(String prefix, String uri) throws SAXException {
    }

    public void endPrefixMapping(String prefix) throws SAXException {
    }

    public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
        if (qName.equals("annotations")) {
            if (this.currentFile.getAbsolutePath().contains("MBh05UdyogaParvan")) {
                qName = "annotations";
            }
            String fn = currentFile.getAbsolutePath();
            int idx = fn.lastIndexOf("\\");
            if (idx == -1) {
                idx = fn.lastIndexOf("/");
            }
            fn = fn.substring(0, idx);
            Integer n = annotationsCount.get(fn);
            if (n == null) {
                n = new Integer(1);
            } else {
                n = new Integer(n.intValue() + 1);
            }
            annotationsCount.put(fn, n);

            String page = atts.getValue("page");
            if (page == null) {
                System.out.format("%s does not have a page attribute\n", currentFile.getAbsolutePath());
            } else if (legalPage == null) {
                System.out.println("Why is legalPage equal to null?");
            } else if (!fudgeEquals(page, legalPage)) {
                if (mismatch.get(legalPage) == null) {
                    mismatch.put(legalPage, true);
                    System.out.format("Directory %s does not match the page attribute '%s' in file %s\n",
                            legalPage, page, xmlfile);
                }
            }
        }
    }

    private boolean fudgeEquals(String page, String legalPage) {
        if (page.equals(legalPage)) {
            return true;
        } else {
            if (page.startsWith("BagavadgItA")) {
                page = page.replaceFirst("BagavadgItA", "BhG");
                return page.equals(legalPage);
            } else if (page.startsWith("SivasahasranAman")) {
                page = page.replaceFirst("SivasahasranAman", "SSN");
                return page.equals(legalPage);
            } else if (page.startsWith("vizRusahasranAman")) {
                page = page.replaceFirst("vizRusahasranAman", "VSN");
                return page.equals(legalPage);

            } else if (page.startsWith("BhP10rasa")) {
                page = page.replaceFirst("BhP10rasa", "BhP10");
                return page.equals(legalPage);
            } else {
                return false;
            }
        }
    }

    public void endElement(String uri, String localName, String qName) throws SAXException {
    }

    public void characters(char[] ch, int start, int length) throws SAXException {
    }

    public class AnnotationInfo implements Comparator<AnnotationInfo> {
        String filename;
        int count;

        public AnnotationInfo () {

        }

        public AnnotationInfo (String filename, int count) {
            this.filename = filename;
            this.count = count;
        }

        public int compare(AnnotationInfo o1, AnnotationInfo o2) {
            return o1.count - o2.count;
        }
    }
}
